/*
 * MikroSDK - MikroE Software Development Kit
 * Copyright (c) 2019, MikroElektronika - www.mikroe.com
 * All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

/*!
 * \file
 *
 */

#include "c3dhall.h"

// ------------------------------------------------------------- PRIVATE MACROS 

#define C3DHALL_DUMMY 0

void c3dhall_cfg_setup ( c3dhall_cfg_t *cfg )
{
    // Communication gpio pins 

    cfg->sck = HAL_PIN_NC;
    cfg->miso = HAL_PIN_NC;
    cfg->mosi = HAL_PIN_NC;
    cfg->cs = HAL_PIN_NC;

    // Additional gpio pins



    cfg->spi_speed = SPI_MEDIUM; 
    cfg->spi_mode = SPI_MODE_1;
}

C3DHALL_RETVAL c3dhall_init ( c3dhall_t *ctx, c3dhall_cfg_t *cfg )
{
    spi_master_config_t spi_cfg;

    spi_master_configure_default( &spi_cfg );
    spi_cfg.mode        = cfg->spi_mode;
    spi_cfg.speed       = cfg->spi_speed;
    spi_cfg.pins.sck    = cfg->sck;
    spi_cfg.pins.miso   = cfg->miso;
    spi_cfg.pins.mosi   = cfg->mosi;
    spi_cfg.chip_select = cfg->cs;

    if (  spi_master_open( &ctx->spi, &spi_cfg ) != SPI_SUCCESS )
    {
        return C3DHALL_INIT_ERROR;
    }

    spi_master_set_dummy_data( &ctx->spi, C3DHALL_DUMMY );

    // Output pins 
    

    // Input pins

    
    // Example: Sets pin [high, low] 

	
	// Example: Get pin state 


    return C3DHALL_OK;

}

void c3dhall_generic_transfer ( c3dhall_t *ctx, spi_master_transfer_data_t *block )
{
    spi_master_start(&ctx->spi);
    Delay_1ms();
    spi_master_transfer(&ctx->spi, block);
    Delay_1ms();
    spi_master_stop(&ctx->spi);    
}

void c3dhall_read_all_data ( c3dhall_t *ctx, c3dhall_all_data_t *all_data )
{
    spi_master_transfer_data_t block;
    uint8_t write_buffer_start[ 8 ];
    uint8_t buffer_read[ 8 ];
    
    write_buffer_start[ 0 ] = C3DHALL_START_COMMUNICATION_BYTE;

    block.tx_buffer = write_buffer_start;
    block.tx_length = 1;
    block.rx_buffer = buffer_read;
    block.rx_length = 8;
   

    c3dhall_generic_transfer( ctx, &block ); 


    all_data->data_angle_a = buffer_read[ 2 ];
    all_data->data_angle_a <<= 8;
    all_data->data_angle_a |= buffer_read[ 1 ];

    all_data->data_angle_b = buffer_read[ 4 ];
    all_data->data_angle_b <<= 8;
    all_data->data_angle_b |= buffer_read[ 3 ];

    all_data->data_error = buffer_read[ 6 ];
    all_data->data_error <<= 8;
    all_data->data_error |= buffer_read[ 5 ];

    all_data->data_sum = buffer_read[ 7 ];
}


uint8_t c3dhall_calculate_angle ( c3dhall_t *ctx, uint16_t data_angle )
{
    uint32_t temp;
    uint8_t result;



    temp = ( uint32_t ) data_angle;
    temp *= 180;
    temp /= 65535;
    
    result = ( uint8_t ) temp;
    
    return result;
}
// ------------------------------------------------------------------------- END

